@@ -22,6 +22,7 @@
 
 #include <gsl/gsl_cblas.h>
 
+
 #include <R.h>
 #include <Rinternals.h>
 #include <R_ext/Print.h>
@@ -30,7 +31,6 @@
 extern "C" {
 #endif	
 	
-    
 	
 	// metaScale
 	void metaScale(int* p, int* n, int* k, double* w, double* m, double* s, int* label, int* method)
@@ -76,8 +76,9 @@ extern "C" {
     
 	void metaHC(int* li, int* lj, double* crit, int* k, int* p, double* w, double* m, double* s)
 	{
+        //dbg::printf("metaHC: alpha=%.4lf", *alpha);
 		mvn_dendro dendro(*k, *p, w, m, s);
-		dendro.hellinger(li, lj, crit);
+		dendro.hellinger_fast(li, lj, crit);
 	}
     
     
@@ -102,57 +103,58 @@ extern "C" {
 		
 	}
     
-    
+
     //
     // call methods
     //
     static void _copyMixtureModel(SEXP res_cluster, double* W, double* M, double* S)
     {
-        int K = INTEGER(R_do_slot(res_cluster, install("K")))[0];
-        int P = INTEGER(Rf_getAttrib(res_cluster, install("P")))[0];
-        cblas_dcopy(K, REAL(R_do_slot(res_cluster, install("w"))), 1, W, 1);
-        const double* mu = REAL(R_do_slot(res_cluster, install("mu")));
-        const double* sigma =  REAL(R_do_slot(res_cluster, install("sigma")));
+        int K = INTEGER(R_do_slot(res_cluster, Rf_install("K")))[0];
+        int P = INTEGER(Rf_getAttrib(res_cluster, Rf_install("P")))[0];
+        cblas_dcopy(K, REAL(R_do_slot(res_cluster, Rf_install("w"))), 1, W, 1);
+        const double* mu = REAL(R_do_slot(res_cluster, Rf_install("mu")));
+        const double* sigma =  REAL(R_do_slot(res_cluster, Rf_install("sigma")));
         
         for( int k=0; k<K; ++k ) {
             cblas_dcopy(P, mu+k, K, M+k*P, 1);
             cblas_dcopy(P*P, sigma+k, K, S+k*P*P, 1);
         }
+       
     }
     static SEXP _ME_ret(int n, int p, int k) 
 	{
         
-		SEXP ret = Rf_protect(allocVector(VECSXP, 12));
-		SEXP names = Rf_protect(allocVector(STRSXP, 12));
+		SEXP ret = Rf_protect(Rf_allocVector(VECSXP, 12));
+		SEXP names = Rf_protect(Rf_allocVector(STRSXP, 12));
 		
-		SET_STRING_ELT(names, 0, mkChar("L"));
-		SET_STRING_ELT(names, 1, mkChar("z"));
-		SET_STRING_ELT(names, 2, mkChar("w"));
-		SET_STRING_ELT(names, 3, mkChar("m"));
-		SET_STRING_ELT(names, 4, mkChar("s"));
-		SET_STRING_ELT(names, 5, mkChar("label"));
-		SET_STRING_ELT(names, 6, mkChar("logLike"));
-		SET_STRING_ELT(names, 7, mkChar("history"));
-		SET_STRING_ELT(names, 8, mkChar("status"));
-		SET_STRING_ELT(names, 9, mkChar("iterations"));
-		SET_STRING_ELT(names, 10, mkChar("tolerance"));
-        SET_STRING_ELT(names, 11, mkChar("normedM"));
+		SET_STRING_ELT(names, 0, Rf_mkChar("L"));
+		SET_STRING_ELT(names, 1, Rf_mkChar("z"));
+		SET_STRING_ELT(names, 2, Rf_mkChar("w"));
+		SET_STRING_ELT(names, 3, Rf_mkChar("m"));
+		SET_STRING_ELT(names, 4, Rf_mkChar("s"));
+		SET_STRING_ELT(names, 5, Rf_mkChar("label"));
+		SET_STRING_ELT(names, 6, Rf_mkChar("logLike"));
+		SET_STRING_ELT(names, 7, Rf_mkChar("history"));
+		SET_STRING_ELT(names, 8, Rf_mkChar("status"));
+		SET_STRING_ELT(names, 9, Rf_mkChar("iterations"));
+		SET_STRING_ELT(names, 10, Rf_mkChar("tolerance"));
+        SET_STRING_ELT(names, 11, Rf_mkChar("normedM"));
         
-		SET_VECTOR_ELT(ret, 0, allocVector(INTSXP, 1));		// out L
-		SET_VECTOR_ELT(ret, 1, allocVector(REALSXP, n*k));	// out z (!not initialzed!)
-		SET_VECTOR_ELT(ret, 2, allocVector(REALSXP, k));	// out w
-		SET_VECTOR_ELT(ret, 3, allocVector(REALSXP, k*p));	// out m
-		SET_VECTOR_ELT(ret, 4, allocVector(REALSXP, k*p*p));// out s
-		SET_VECTOR_ELT(ret, 5, allocVector(INTSXP, n));		// out label
-		SET_VECTOR_ELT(ret, 6, allocVector(REALSXP, 3));	// out logLike
-		SET_VECTOR_ELT(ret, 7, allocVector(INTSXP, k));		// out history
-		SET_VECTOR_ELT(ret, 8, allocVector(INTSXP, 1));		// out status
-		SET_VECTOR_ELT(ret, 9, allocVector(INTSXP, 1));		// out iteratioms
-		SET_VECTOR_ELT(ret, 10, allocVector(REALSXP, 1));	// out tolerance
+		SET_VECTOR_ELT(ret, 0, Rf_allocVector(INTSXP, 1));		// out L
+		SET_VECTOR_ELT(ret, 1, Rf_allocVector(REALSXP, n*k));	// out z (!not initialzed!)
+		SET_VECTOR_ELT(ret, 2, Rf_allocVector(REALSXP, k));	// out w
+		SET_VECTOR_ELT(ret, 3, Rf_allocVector(REALSXP, k*p));	// out m
+		SET_VECTOR_ELT(ret, 4, Rf_allocVector(REALSXP, k*p*p));// out s
+		SET_VECTOR_ELT(ret, 5, Rf_allocVector(INTSXP, n));		// out label
+		SET_VECTOR_ELT(ret, 6, Rf_allocVector(REALSXP, 4));	// out logLike
+		SET_VECTOR_ELT(ret, 7, Rf_allocVector(INTSXP, k));		// out history
+		SET_VECTOR_ELT(ret, 8, Rf_allocVector(INTSXP, 1));		// out status
+		SET_VECTOR_ELT(ret, 9, Rf_allocVector(INTSXP, 1));		// out iteratioms
+		SET_VECTOR_ELT(ret, 10, Rf_allocVector(REALSXP, 1));	// out tolerance
 		
-        SET_VECTOR_ELT(ret, 11, allocVector(REALSXP, n*p));   //normedM
+        SET_VECTOR_ELT(ret, 11, Rf_allocVector(REALSXP, n*p));   //normedM
         
-    	setAttrib(ret, R_NamesSymbol, names);
+        Rf_setAttrib(ret, R_NamesSymbol, names);
 		
 		Rf_unprotect(1);	// unproctedt names
 		
@@ -165,7 +167,7 @@ extern "C" {
                      SEXP label, SEXP max_iter, SEXP max_tol, SEXP method, 
                      SEXP bias, SEXP alpha, SEXP min_g)
 	{
-		int status = 0;
+		int status = 0, L = INTEGER(K)[0];
        
         int iterations = INTEGER(max_iter)[0];
         double tolerance = REAL(max_tol)[0];
@@ -181,70 +183,82 @@ extern "C" {
        
 		switch(INTEGER(method)[0]) {
             case 1:		// bc EM: no weights
-				em.start(INTEGER(label), false);
+				L = em.start(INTEGER(label), false);
 				status = em.bc_maximize(iterations, tolerance);
 				break;
 			case 2:		// bc EM-T: classification no weights
-				em.start(INTEGER(label), false);
+				L = em.start(INTEGER(label), false);
 				status = em.bc_classify(iterations, tolerance, INTEGER(min_g)[0]);
 				break;
-            case 3:     // kl EM: no weights
-                em.start(INTEGER(label), false);
-                status = em.kl_maximize(iterations, tolerance);
-                break;
-            case 4:     // kl EM-T: no weights
-                em.start(INTEGER(label), false);
-                status = em.kl_classify(iterations, tolerance, INTEGER(min_g)[0]);
-                break;
-            
+          
             case 10:    // bc EM: weights
-				em.start(INTEGER(label), true);
+            case 100:   // kann eigentlich weg = 10, macht ja nichts anderes
+				L = em.start(INTEGER(label), true);
 				status = em.bc_maximize(iterations, tolerance);
 				break;
+         
+                
 			case 20:	// bc EM-T: classification with weights
-				em.start(INTEGER(label), true);
+            case 200:   // bc EM-T: classification with weights / final no weights: macht eher keine sinn
+            case 300:   // trail for changed final, kann wieder weg == 20
+				L = em.start(INTEGER(label), true);
 				status = em.bc_classify(iterations, tolerance, INTEGER(min_g)[0]);
 				break;
           
             case 23:    // bc EM-T: classification with weights were the labeling of min_g cluster remains unchanged
-                em.start(INTEGER(label), true);
+                L = em.start(INTEGER(label), true);
                 status = em.bc_fixedN_classify(iterations, tolerance, INTEGER(min_g)[0]);
                 break;
-            
-            case 30:    // kl EM: weights
-                em.start(INTEGER(label), true);
-                status = em.kl_maximize(iterations, tolerance);
+                
+                
+            default:
+                L = em.start(INTEGER(label), false);
+                status = em.bc_maximize(iterations, tolerance);
                 break;
-            case 40:    // kl EM-T: classification with weights
-                em.start(INTEGER(label), true);
-                status = em.kl_classify(iterations, tolerance, INTEGER(min_g)[0]);
-                break;
-            case 43:    // kl EM-T: classification with weights were the labeling of min_g cluster remains unchanged
-                em.start(INTEGER(label), true);
-                status = em.kl_fixedN_classify(iterations, tolerance, INTEGER(min_g)[0]);
-                break;
-            
-			default:
-				em.start(INTEGER(label), false);
-				status = em.kl_minimize(iterations, tolerance);
-				break;
 		}
-		INTEGER(VECTOR_ELT(ret,8))[0] = status;		
+        
+		INTEGER(VECTOR_ELT(ret,8))[0] = status;
 		INTEGER(VECTOR_ELT(ret,9))[0] = iterations;
 		REAL(VECTOR_ELT(ret,10))[0] = tolerance;
         
-      
-		INTEGER(VECTOR_ELT(ret,0))[0] = em.final(INTEGER(VECTOR_ELT(ret,5)),
-                                                 REAL(VECTOR_ELT(ret,6)), 
-                                                 INTEGER(VECTOR_ELT(ret,7)) );
-        
+    
+        switch(INTEGER(method)[0]) {
+            case 300:   // final with weights, remove empty
+            default:    // old final with weights, can contain finally empty
+                INTEGER(VECTOR_ELT(ret,0))[0] = em.final3(INTEGER(VECTOR_ELT(ret,5)),
+                                                        REAL(VECTOR_ELT(ret,6)),
+                                                        INTEGER(VECTOR_ELT(ret,7)) );
+                break;
+                
+            case 200:   // final not weights, remove empty
+                INTEGER(VECTOR_ELT(ret,0))[0] = em.final2(INTEGER(VECTOR_ELT(ret,5)),
+                                                        REAL(VECTOR_ELT(ret,6)),
+                                                        INTEGER(VECTOR_ELT(ret,7)) );
+                break;
+                
+           /*
+            default:
+                INTEGER(VECTOR_ELT(ret,0))[0] = em.final1(INTEGER(VECTOR_ELT(ret,5)),
+                                                        REAL(VECTOR_ELT(ret,6)),
+                                                        INTEGER(VECTOR_ELT(ret,7)) );
+                break;
+            */
+        }
         
+        const double* logLike = REAL(VECTOR_ELT(ret,6));
+        dbg::printf("EM[%d] (%d obs, %d cls, %d iter) => %d cluster (%.0lf|%.0lf)",
+                    INTEGER(method)[0], INTEGER(N)[0], L,
+                    iterations, INTEGER(VECTOR_ELT(ret,0))[0], logLike[3], logLike[2]-logLike[3]
+                    );
+                    
         Rf_unprotect(1);	// unprocted ret
                                                  
         return ret;
         
 	}
     
+
+
     // >> SON clustering
     SEXP call_SON_combineClustering(SEXP res_model, SEXP res_sample,
                                     SEXP map_cluster, SEXP use_cluster,
@@ -257,9 +271,9 @@ extern "C" {
                                     )
     {
         
-        int G = INTEGER(R_do_slot(res_model, install("K")))[0];
-        int K = INTEGER(Rf_getAttrib(res_sample, install("K")))[0];
-        int P = INTEGER(Rf_getAttrib(res_model, install("P")))[0];
+        int G = INTEGER(R_do_slot(res_model, Rf_install("K")))[0];
+        int K = INTEGER(Rf_getAttrib(res_sample, Rf_install("K")))[0];
+        int P = INTEGER(Rf_getAttrib(res_model, Rf_install("P")))[0];
         int N = G+K;
         
         dbg::printf("SON_combineClustering: G=%d, K=%d, P=%d, N=%d", G, K, P, N);
@@ -273,10 +287,10 @@ extern "C" {
         double* nS = new double[N*P*P];
         
         
-        cblas_dcopy(G, REAL(R_do_slot(res_model, install("evts"))), 1, nEvts, 1);
+        cblas_dcopy(G, REAL(R_do_slot(res_model, Rf_install("evts"))), 1, nEvts, 1);
         _copyMixtureModel(res_model, nW, nM, nS);
    
-        cblas_dcopy(K, REAL(R_do_slot(res_sample, install("evts"))), 1, nEvts+G, 1);
+        cblas_dcopy(K, REAL(R_do_slot(res_sample, Rf_install("evts"))), 1, nEvts+G, 1);
         _copyMixtureModel(res_sample, nW+G, nM+G*P, nS+G*P*P);
         
         int* label = INTEGER(VECTOR_ELT(ret,5));
@@ -320,11 +334,13 @@ extern "C" {
             
         }
         
+        int SON_norm = INTEGER(SON_method)[0];
+        
         meta_SON son(P,
                      G, nW, nEvts, mappedM, nS,
                      K, nW+G, nEvts+G, clusterM, nS+G*P*P,
                      normedM,
-                     REAL(alpha)[0],
+                     REAL(alpha)[0], 1,     // old blur mode
                      gTrace, kTrace, 0);
         em_meta em(N, P, G, nEvts, nM, nS,
                    z, w, mappedM, s,   // output
@@ -335,12 +351,6 @@ extern "C" {
         // em with scaled model sigma too?
         //for( int cycle=0; cycle < INTEGER(meta_cycles)[0]; ++cycle) {
         {
-            // 2023.06.15: only for Trails
-            //int SON_norm = INTEGER(meta_cycles)[0];
-            int SON_norm = INTEGER(SON_method)[0];
-            
-            // Rprintf("combineClustring %d\n", SON_norm);
-            //int SON_norm = 1;
             //dbg::printf("meta cycle %d", cycle);
             // re-label map_cluster?? obsolete because fixedN clustering
             // map.cluster <- unique(label[map_cluster])
@@ -394,9 +404,7 @@ extern "C" {
             // keep model cluster fixed
             em.bc_fixedN_classify(max_iteration, max_tolerance, G);
             
-            //double logLike[3];
-            
-            INTEGER(VECTOR_ELT(ret,0))[0] = em.final(label,
+            INTEGER(VECTOR_ELT(ret,0))[0] = em.final1(label,
                                                      REAL(VECTOR_ELT(ret,6)),
                                                      INTEGER(VECTOR_ELT(ret,7)));
             
@@ -436,7 +444,7 @@ extern "C" {
         }
         
         delete[] clusterM;
-        //delete[] mapS;
+
         delete[] nS;
         delete[] nW;
         delete[] nEvts;
@@ -451,12 +459,12 @@ extern "C" {
     SEXP call_SON_normalize(SEXP res_model,
                             SEXP n, SEXP k, SEXP w, SEXP m, SEXP s,
                             SEXP alpha, SEXP scale_factor, SEXP scale_steps,
-                            SEXP meta_iter, SEXP meta_tol,
+                            // SEXP meta_iter, SEXP meta_tol, // obsolete!!!
                             SEXP SON_cycles, SEXP SON_rlen, SEXP SON_deltas, SEXP SON_blurring
                             )
     {
-        int P = INTEGER(Rf_getAttrib(res_model, install("P")))[0];
-        int G = INTEGER(Rf_getAttrib(res_model, install("K")))[0];
+        int P = INTEGER(Rf_getAttrib(res_model, Rf_install("P")))[0];
+        int G = INTEGER(Rf_getAttrib(res_model, Rf_install("K")))[0];
         
         double* gW = new double[G];
         double* gM = new double[G*P];
@@ -464,23 +472,20 @@ extern "C" {
         
         _copyMixtureModel(res_model, gW, gM, gS);
         
-        int  N = INTEGER(n)[0];
-        const int* K = INTEGER(k);
-        const double* W = REAL(w);
-        const double* M = REAL(m);
-        const double* S = REAL(s);
+        int  N = INTEGER(n)[0];     // number of samples
+        const int* K = INTEGER(k);  // array with number of sample cluster
+        const double* W = REAL(w);  // sample cluster cell-event count
+        const double* M = REAL(m);  // sample cluster mean
+        const double* S = REAL(s);  // sample cluster co-variance
         //const double* tM = REAL(tm);
         int totK = 0;
         for( int i=0; i<N; ++i)
             totK += K[i];
         
-        int*  label = new int[totK];
-        double* z = new double[totK*G];
         
-        SEXP ret = Rf_protect(allocVector(REALSXP, totK*P));
+        SEXP ret = Rf_protect(Rf_allocVector(REALSXP, totK*P));
         
         double* normedM = REAL(ret);
-        //int* nLabel = label;
         const double* nW = W;
         const double* nM = M;
         const double* nS = S;
@@ -489,45 +494,82 @@ extern "C" {
             // norm nth sample
             int L = G;
             
+           
             /*
-            int max_iteration = INTEGER(meta_iter)[0];
-            double max_tolerance = REAL(meta_tol)[0];
-            
-            if( max_iteration > 0 ) {
-                em_meta em(totK, P, G, W, M, S,
-                           z, gW, gM, gS,   // output
-                           0, REAL(alpha)[0] ); // BIAS not used in here
-                
-                memcpy(label, INTEGER(Rf_getAttrib(res_model, install("label"))), totK*sizeof(int));
-                memset(nLabel, 0, K[n]*sizeof(int));
-                
-                em.bc_maximize(max_iteration, max_tolerance);
-                double logLike[3];
-                L = em.final(label, logLike, 0);
+            int traceK[3];
+            traceK[2] = -1;
+            if( n == 24 ) {
+                traceK[0] = 84;
+                traceK[1] = 86;
             }
-            */
+            else {
+                traceK[0] = -1;
+            }
+            
+            int traceG[3];
+            traceG[0] = -1;
+            
+            if( n == 24 ) {
+                 traceG[0] = 0;
+                 traceG[1] = -1;
+             }
+             else {
+                 traceG[0] = -1;
+             }
+             */
             
             meta_SON son(P, L, gW, gW, gM, gS,
                          K[n], nW, nW, nM, nS,
                          normedM,
-                         REAL(alpha)[0], 0, 0, FALSE);
+                         REAL(alpha)[0], 0, // fast blur
+                         0, 0, FALSE);
             // mayby scale first
             if(INTEGER(scale_steps)[0] > 0)
                 son.scaleStep(REAL(scale_factor)[0], INTEGER(scale_steps)[0] );
             // map
+            // 2024.09.19: normStep2 corrected?
             son.normStep2( 0, 0,
                          INTEGER(SON_cycles)[0], INTEGER(SON_rlen)[0],
                          REAL(SON_deltas), REAL(SON_blurring));
             
-            //nLabel += K[n];
             nW += K[n];
             nM += K[n]*P;
             nS += K[n]*P*P;
             normedM += K[n]*P;
         }
+
+        // re-location normalized center to orignal
+        // 2024.09.10: wirklich???
+        // habe die idee dazu vergessen
+        const double* label = REAL(R_do_slot(res_model, Rf_install("label")));
+        normedM = REAL(ret);
         
-        delete[] z;
-        delete[] label;
+        for( int j = 0; j < G; ++j ) {
+            double* diff_m = new double[P];
+            double comp_w = 0;
+            memset(diff_m, 0, P*sizeof(double));
+            
+            for( int i=0; i < totK; ++i) {
+                if( label[i] == j+1 ) {
+                    cblas_daxpy(P, W[i], M+i*P, 1, diff_m, 1 );
+                    cblas_daxpy(P, -W[i], normedM+i*P, 1, diff_m, 1 );
+                    comp_w += W[i];
+                }
+            }
+            if( comp_w == 0.0 ) {
+                dbg::printf("SON: no obs for cls %d", j);
+                continue;
+            }
+            
+            cblas_dscal(P, 1.0/comp_w, diff_m, 1);
+            for( int i=0; i < totK; ++i) {
+                if( label[i] == j+1 ) {
+                    cblas_daxpy(P, 1.0, diff_m, 1, normedM+i*P, 1 );
+                }
+            }
+        } // for comp j
+        //
+
         delete[] gW;
         delete[] gM;
         delete[] gS;
@@ -543,15 +585,15 @@ extern "C" {
     SEXP call_mvnDist(SEXP P, SEXP K, SEXP W, SEXP M, SEXP S)
     {
         int k = INTEGER(K)[0];
-        SEXP ret = Rf_protect(allocVector(VECSXP, k*(k-1)/2));
+        SEXP ret = Rf_protect(Rf_allocVector(VECSXP, k*(k-1)/2));
         
         dist_mvn dist(INTEGER(P)[0], INTEGER(K)[0],
                  REAL(W), REAL(M), REAL(S));
         
         dist.hellinger(REAL(ret));
-        Rf_setAttrib(ret,install("Size"), Rf_duplicate(K));
-        Rf_setAttrib(ret,install("Diag"), Rf_ScalarLogical(0));
-        Rf_setAttrib(ret,install("Upper"), Rf_ScalarLogical(0));
+        Rf_setAttrib(ret,Rf_install("Size"), Rf_duplicate(K));
+        Rf_setAttrib(ret,Rf_install("Diag"), Rf_ScalarLogical(0));
+        Rf_setAttrib(ret,Rf_install("Upper"), Rf_ScalarLogical(0));
         Rf_setAttrib(ret,R_ClassSymbol, Rf_mkString("dist"));
         Rf_unprotect(1);
         
