@@ -18,7 +18,6 @@
 #include "sub_mvn.h"
 #include "util.h"
 
-
 #include <R.h>
 #include <Rinternals.h>
 #include <R_ext/Print.h>
@@ -58,36 +57,38 @@ extern "C" {
 	 logLike:	to store 3 likelihood values (observation likelihood, classification likelihood, ...)
 	 */
 	
-	static SEXP _ME_ret(int n, int p, int k) 
+	static SEXP _ME_ret(int n, int p, int k)
 	{
-		SEXP ret = Rf_protect(allocVector(VECSXP, 11));
-		SEXP names = Rf_protect(allocVector(STRSXP, 11));
+        
+		SEXP ret = Rf_protect(Rf_allocVector(VECSXP, 11));
+		SEXP names = Rf_protect(Rf_allocVector(STRSXP, 11));
 		
-		SET_STRING_ELT(names, 0, mkChar("L"));
-		SET_STRING_ELT(names, 1, mkChar("z"));
-		SET_STRING_ELT(names, 2, mkChar("w"));
-		SET_STRING_ELT(names, 3, mkChar("m"));
-		SET_STRING_ELT(names, 4, mkChar("s"));
-		SET_STRING_ELT(names, 5, mkChar("label"));
-		SET_STRING_ELT(names, 6, mkChar("logLike"));
-		SET_STRING_ELT(names, 7, mkChar("history"));
-		SET_STRING_ELT(names, 8, mkChar("status"));
-		SET_STRING_ELT(names, 9, mkChar("iterations"));
-		SET_STRING_ELT(names, 10, mkChar("tolerance"));
+		SET_STRING_ELT(names, 0, Rf_mkChar("L"));
+		SET_STRING_ELT(names, 1, Rf_mkChar("z"));
+		SET_STRING_ELT(names, 2, Rf_mkChar("w"));
+		SET_STRING_ELT(names, 3, Rf_mkChar("m"));
+		SET_STRING_ELT(names, 4, Rf_mkChar("s"));
+		SET_STRING_ELT(names, 5, Rf_mkChar("label"));
+		SET_STRING_ELT(names, 6, Rf_mkChar("logLike"));
+		SET_STRING_ELT(names, 7, Rf_mkChar("history"));
+		SET_STRING_ELT(names, 8, Rf_mkChar("status"));
+		SET_STRING_ELT(names, 9, Rf_mkChar("iterations"));
+		SET_STRING_ELT(names, 10, Rf_mkChar("tolerance"));
 		
-		SET_VECTOR_ELT(ret, 0, allocVector(INTSXP, 1));		// out L
-		SET_VECTOR_ELT(ret, 1, allocVector(REALSXP, n*k));	// out z (!not initialzed!)
-		SET_VECTOR_ELT(ret, 2, allocVector(REALSXP, k));	// out w
-		SET_VECTOR_ELT(ret, 3, allocVector(REALSXP, k*p));	// out m
-		SET_VECTOR_ELT(ret, 4, allocVector(REALSXP, k*p*p));// out s
-		SET_VECTOR_ELT(ret, 5, allocVector(INTSXP, n));		// out label
-		SET_VECTOR_ELT(ret, 6, allocVector(REALSXP, 3));	// out logLike
-		SET_VECTOR_ELT(ret, 7, allocVector(INTSXP, k));		// out history
-		SET_VECTOR_ELT(ret, 8, allocVector(INTSXP, 1));		// out status
-		SET_VECTOR_ELT(ret, 9, allocVector(INTSXP, 1));		// out iterations
-		SET_VECTOR_ELT(ret, 10, allocVector(REALSXP, 1));	// out tollerance
+		SET_VECTOR_ELT(ret, 0, Rf_allocVector(INTSXP, 1));		// out L
+        //SET_VECTOR_ELT(ret, 1, Rf_allocVector(REALSXP, n*k));  // out z (!not initialized!)
+		SET_VECTOR_ELT(ret, 1, Rf_allocVector(REALSXP, R_xlen_t(n)*R_xlen_t(k)));	// out z (!not initialized!)
+		SET_VECTOR_ELT(ret, 2, Rf_allocVector(REALSXP, k));	// out w
+		SET_VECTOR_ELT(ret, 3, Rf_allocVector(REALSXP, k*p));	// out m
+		SET_VECTOR_ELT(ret, 4, Rf_allocVector(REALSXP, k*p*p));// out s
+		SET_VECTOR_ELT(ret, 5, Rf_allocVector(INTSXP, n));		// out label
+		SET_VECTOR_ELT(ret, 6, Rf_allocVector(REALSXP, 3));	// out logLike
+		SET_VECTOR_ELT(ret, 7, Rf_allocVector(INTSXP, k));		// out history
+		SET_VECTOR_ELT(ret, 8, Rf_allocVector(INTSXP, 1));		// out status
+		SET_VECTOR_ELT(ret, 9, Rf_allocVector(INTSXP, 1));		// out iterations
+		SET_VECTOR_ELT(ret, 10, Rf_allocVector(REALSXP, 1));	// out tollerance
 		        
-		setAttrib(ret, R_NamesSymbol, names);
+        Rf_setAttrib(ret, R_NamesSymbol, names);
 		
 		Rf_unprotect(1);	// unproctect names
 		
@@ -97,34 +98,34 @@ extern "C" {
     
     static SEXP _M_ret(int n, int p, int k, SEXP label) 
 	{
-		SEXP ret = Rf_protect(allocVector(VECSXP, 11));
-		SEXP names = Rf_protect(allocVector(STRSXP, 11));
+		SEXP ret = Rf_protect(Rf_allocVector(VECSXP, 11));
+		SEXP names = Rf_protect(Rf_allocVector(STRSXP, 11));
 		
-		SET_STRING_ELT(names, 0, mkChar("L"));
-		SET_STRING_ELT(names, 1, mkChar("z"));
-		SET_STRING_ELT(names, 2, mkChar("w"));
-		SET_STRING_ELT(names, 3, mkChar("m"));
-		SET_STRING_ELT(names, 4, mkChar("s"));
-		SET_STRING_ELT(names, 5, mkChar("label"));
-		SET_STRING_ELT(names, 6, mkChar("logLike"));
-		SET_STRING_ELT(names, 7, mkChar("history"));
-		SET_STRING_ELT(names, 8, mkChar("status"));
-		SET_STRING_ELT(names, 9, mkChar("iterations"));
-		SET_STRING_ELT(names, 10, mkChar("tolerance"));
+		SET_STRING_ELT(names, 0, Rf_mkChar("L"));
+		SET_STRING_ELT(names, 1, Rf_mkChar("z"));
+		SET_STRING_ELT(names, 2, Rf_mkChar("w"));
+		SET_STRING_ELT(names, 3, Rf_mkChar("m"));
+		SET_STRING_ELT(names, 4, Rf_mkChar("s"));
+		SET_STRING_ELT(names, 5, Rf_mkChar("label"));
+		SET_STRING_ELT(names, 6, Rf_mkChar("logLike"));
+		SET_STRING_ELT(names, 7, Rf_mkChar("history"));
+		SET_STRING_ELT(names, 8, Rf_mkChar("status"));
+		SET_STRING_ELT(names, 9, Rf_mkChar("iterations"));
+		SET_STRING_ELT(names, 10, Rf_mkChar("tolerance"));
 		
-		SET_VECTOR_ELT(ret, 0, allocVector(INTSXP, 1));		// out L
-		SET_VECTOR_ELT(ret, 1, allocVector(REALSXP, 0));	// out z (=NULL)
-		SET_VECTOR_ELT(ret, 2, allocVector(REALSXP, k));	// out w
-		SET_VECTOR_ELT(ret, 3, allocVector(REALSXP, k*p));	// out m
-		SET_VECTOR_ELT(ret, 4, allocVector(REALSXP, k*p*p));// out s
+		SET_VECTOR_ELT(ret, 0, Rf_allocVector(INTSXP, 1));		// out L
+		SET_VECTOR_ELT(ret, 1, Rf_allocVector(REALSXP, 0));	// out z (=NULL)
+		SET_VECTOR_ELT(ret, 2, Rf_allocVector(REALSXP, k));	// out w
+		SET_VECTOR_ELT(ret, 3, Rf_allocVector(REALSXP, k*p));	// out m
+		SET_VECTOR_ELT(ret, 4, Rf_allocVector(REALSXP, k*p*p));// out s
 		SET_VECTOR_ELT(ret, 5, Rf_duplicate(label));		// out label
-		SET_VECTOR_ELT(ret, 6, allocVector(REALSXP, 3));	// out logLike
-		SET_VECTOR_ELT(ret, 7, allocVector(INTSXP, k));     // out history
-		SET_VECTOR_ELT(ret, 8, allocVector(INTSXP, 1));		// out status
-		SET_VECTOR_ELT(ret, 9, allocVector(INTSXP, 1));		// out iterations
-		SET_VECTOR_ELT(ret, 10, allocVector(REALSXP, 1));	// out tollerance
+		SET_VECTOR_ELT(ret, 6, Rf_allocVector(REALSXP, 3));	// out logLike
+		SET_VECTOR_ELT(ret, 7, Rf_allocVector(INTSXP, k));     // out history
+		SET_VECTOR_ELT(ret, 8, Rf_allocVector(INTSXP, 1));		// out status
+		SET_VECTOR_ELT(ret, 9, Rf_allocVector(INTSXP, 1));		// out iterations
+		SET_VECTOR_ELT(ret, 10, Rf_allocVector(REALSXP, 1));	// out tollerance
         
-		setAttrib(ret, R_NamesSymbol, names);
+        Rf_setAttrib(ret, R_NamesSymbol, names);
 		
 		Rf_unprotect(1);	// unproctect names
 		
@@ -134,34 +135,35 @@ extern "C" {
 	
 	static SEXP _EM_ret(int n, int p, int k, SEXP w, SEXP m, SEXP s) 
 	{
-		SEXP ret = Rf_protect(allocVector(VECSXP, 11));
-		SEXP names = Rf_protect(allocVector(STRSXP, 11));
+		SEXP ret = Rf_protect(Rf_allocVector(VECSXP, 11));
+		SEXP names = Rf_protect(Rf_allocVector(STRSXP, 11));
 		
-		SET_STRING_ELT(names, 0, mkChar("L"));
-		SET_STRING_ELT(names, 1, mkChar("z"));
-		SET_STRING_ELT(names, 2, mkChar("w"));
-		SET_STRING_ELT(names, 3, mkChar("m"));
-		SET_STRING_ELT(names, 4, mkChar("s"));
-		SET_STRING_ELT(names, 5, mkChar("label"));
-		SET_STRING_ELT(names, 6, mkChar("logLike"));
-		SET_STRING_ELT(names, 7, mkChar("history"));
-		SET_STRING_ELT(names, 8, mkChar("status"));
-		SET_STRING_ELT(names, 9, mkChar("iterations"));
-		SET_STRING_ELT(names, 10, mkChar("tolerance"));
+		SET_STRING_ELT(names, 0, Rf_mkChar("L"));
+		SET_STRING_ELT(names, 1, Rf_mkChar("z"));
+		SET_STRING_ELT(names, 2, Rf_mkChar("w"));
+		SET_STRING_ELT(names, 3, Rf_mkChar("m"));
+		SET_STRING_ELT(names, 4, Rf_mkChar("s"));
+		SET_STRING_ELT(names, 5, Rf_mkChar("label"));
+		SET_STRING_ELT(names, 6, Rf_mkChar("logLike"));
+		SET_STRING_ELT(names, 7, Rf_mkChar("history"));
+		SET_STRING_ELT(names, 8, Rf_mkChar("status"));
+		SET_STRING_ELT(names, 9, Rf_mkChar("iterations"));
+		SET_STRING_ELT(names, 10, Rf_mkChar("tolerance"));
 		
-		SET_VECTOR_ELT(ret, 0, allocVector(INTSXP, 1));		// out L
-		SET_VECTOR_ELT(ret, 1, allocVector(REALSXP, n*k));	// out z (!not initialized!)
-		SET_VECTOR_ELT(ret, 2, Rf_duplicate(w));			// in/out w
+		SET_VECTOR_ELT(ret, 0, Rf_allocVector(INTSXP, 1));		// out L
+        SET_VECTOR_ELT(ret, 1, Rf_allocVector(REALSXP, n*k));  // out z (!not initialized!)
+        SET_VECTOR_ELT(ret, 1, Rf_allocVector(REALSXP, R_xlen_t(n)*R_xlen_t(k)));	// out z (!not initialized!)
+     	SET_VECTOR_ELT(ret, 2, Rf_duplicate(w));			// in/out w
 		SET_VECTOR_ELT(ret, 3, Rf_duplicate(m));			// in/out m
 		SET_VECTOR_ELT(ret, 4, Rf_duplicate(s));			// in/out s
-		SET_VECTOR_ELT(ret, 5, allocVector(INTSXP,n));		// out label
-		SET_VECTOR_ELT(ret, 6, allocVector(REALSXP, 3));	// out logLike
-		SET_VECTOR_ELT(ret, 7, allocVector(INTSXP, k));		// summaryout histroy
-		SET_VECTOR_ELT(ret, 8, allocVector(INTSXP, 1));		// out status
-		SET_VECTOR_ELT(ret, 9, allocVector(INTSXP, 1));		// out iterations
-		SET_VECTOR_ELT(ret, 10, allocVector(REALSXP, 1));	// out tolerance
+		SET_VECTOR_ELT(ret, 5, Rf_allocVector(INTSXP,n));		// out label
+		SET_VECTOR_ELT(ret, 6, Rf_allocVector(REALSXP, 3));	// out logLike
+		SET_VECTOR_ELT(ret, 7, Rf_allocVector(INTSXP, k));		// summaryout histroy
+		SET_VECTOR_ELT(ret, 8, Rf_allocVector(INTSXP, 1));		// out status
+		SET_VECTOR_ELT(ret, 9, Rf_allocVector(INTSXP, 1));		// out iterations
+		SET_VECTOR_ELT(ret, 10, Rf_allocVector(REALSXP, 1));	// out tolerance
 		
-		setAttrib(ret, R_NamesSymbol, names);
+        Rf_setAttrib(ret, R_NamesSymbol, names);
 
 		Rf_unprotect(1);	// unprotect names
 		
@@ -185,7 +187,7 @@ extern "C" {
 		
 		em_gaussian em(INTEGER(N)[0], INTEGER(P)[0], INTEGER(K)[0], REAL(y), 
                        REAL(VECTOR_ELT(ret,1)), REAL(VECTOR_ELT(ret,2)), REAL(VECTOR_ELT(ret,3)), REAL(VECTOR_ELT(ret,4)), 
-                       (isReal(t) && Rf_length(t) > 0 ) ? REAL(t) : 0, 0.0);
+                       (Rf_isReal(t) && Rf_length(t) > 0 ) ? REAL(t) : 0, 0.0);
 		
 		status = em.start(INTEGER(label));
 		if( status == 0 ) {
@@ -214,7 +216,7 @@ extern "C" {
    	SEXP ret = _M_ret(INTEGER(N)[0], INTEGER(P)[0], INTEGER(K)[0], label);
   		em_gaussian em(INTEGER(N)[0], INTEGER(P)[0], INTEGER(K)[0], REAL(y), 
 					   0, REAL(VECTOR_ELT(ret,2)), REAL(VECTOR_ELT(ret,3)), REAL(VECTOR_ELT(ret,4)), 
-					   (isReal(t) && Rf_length(t) > 0) ? REAL(t) : 0);	
+					   (Rf_isReal(t) && Rf_length(t) > 0) ? REAL(t) : 0);
 		
   		status = em.build(INTEGER(label), REAL(VECTOR_ELT(ret,6)),INTEGER(VECTOR_ELT(ret,7)));
         INTEGER(VECTOR_ELT(ret,0))[0] =  INTEGER(K)[0];
@@ -241,7 +243,7 @@ extern "C" {
 		
 		em_gaussian em(INTEGER(N)[0], INTEGER(P)[0], INTEGER(K)[0], REAL(y), 
 					   REAL(VECTOR_ELT(ret,1)), REAL(VECTOR_ELT(ret,2)), REAL(VECTOR_ELT(ret,3)), REAL(VECTOR_ELT(ret,4)), 
-					   (isReal(t) && Rf_length(t) > 0) ? REAL(t) : 0, REAL(bias)[0] );
+					   (Rf_isReal(t) && Rf_length(t) > 0) ? REAL(t) : 0, REAL(bias)[0] );
 		
 		status = em.start(INTEGER(label));
 		if( status == 0 ) {
@@ -276,7 +278,7 @@ extern "C" {
 		
 		em_gaussian em(INTEGER(N)[0], INTEGER(P)[0], INTEGER(K)[0], REAL(y), 
 					   REAL(VECTOR_ELT(ret,1)), REAL(VECTOR_ELT(ret,2)), REAL(VECTOR_ELT(ret,3)) , REAL(VECTOR_ELT(ret,4)), 
-					   (isReal(t) && Rf_length(t) > 0) ? REAL(t) : 0);	
+					   (Rf_isReal(t) && Rf_length(t) > 0) ? REAL(t) : 0);
 		
 		status = em.start(0);
 		if( status == 0 ) {
@@ -312,7 +314,7 @@ extern "C" {
 		
 		em_gaussian em(INTEGER(N)[0], INTEGER(P)[0], INTEGER(K)[0], REAL(y), 
 					   REAL(VECTOR_ELT(ret,1)), REAL(VECTOR_ELT(ret,2)), REAL(VECTOR_ELT(ret,3)), REAL(VECTOR_ELT(ret,4)), 
-					   (isReal(t) && Rf_length(t) > 0) ? REAL(t) : 0, REAL(bias)[0]);	
+					   (Rf_isReal(t) && Rf_length(t) > 0) ? REAL(t) : 0, REAL(bias)[0]);
 		
 		status = em.start(0);
 		if( status == 0 ) {
@@ -343,7 +345,7 @@ extern "C" {
 
 		em_gaussian em(INTEGER(N)[0], INTEGER(P)[0], INTEGER(K)[0], REAL(y), 
 					   REAL(VECTOR_ELT(ret,1)), REAL(VECTOR_ELT(ret,2)), REAL(VECTOR_ELT(ret,3)), REAL(VECTOR_ELT(ret,4)), 
-					   (isReal(t) && Rf_length(t) > 0) ? REAL(t) : 0);	
+					   (Rf_isReal(t) && Rf_length(t) > 0) ? REAL(t) : 0);
 		
 		status = em.start(0);
 		if( status == 0 ) {
@@ -379,7 +381,7 @@ extern "C" {
 		
 		em_mvt em(INTEGER(N)[0], INTEGER(P)[0], INTEGER(K)[0], REAL(y), 
 				  REAL(VECTOR_ELT(ret,1)), REAL(VECTOR_ELT(ret,2)), REAL(VECTOR_ELT(ret,3)), REAL(VECTOR_ELT(ret,4)), 
-				  MVT_Nu, (isReal(t) && Rf_length(t) > 0) ? REAL(t) : 0, 0.0);
+				  MVT_Nu, (Rf_isReal(t) && Rf_length(t) > 0) ? REAL(t) : 0, 0.0);
 		
 		status = em.start(INTEGER(label));
 		if( status == 0 ) {
@@ -413,7 +415,7 @@ extern "C" {
 		
 		em_mvt em(INTEGER(N)[0], INTEGER(P)[0], INTEGER(K)[0], REAL(y), 
 				  REAL(VECTOR_ELT(ret,1)), REAL(VECTOR_ELT(ret,2)), REAL(VECTOR_ELT(ret,3)), REAL(VECTOR_ELT(ret,4)), 
-				  MVT_Nu, (isReal(t) && Rf_length(t) > 0 ) ? REAL(t) : 0, REAL(bias)[0]);
+				  MVT_Nu, (Rf_isReal(t) && Rf_length(t) > 0 ) ? REAL(t) : 0, REAL(bias)[0]);
 		
 		status = em.start(INTEGER(label));
 		if( status == 0 ) {
@@ -443,7 +445,7 @@ extern "C" {
         SEXP ret = _M_ret(INTEGER(N)[0], INTEGER(P)[0], INTEGER(K)[0], label);
         em_mvt em(INTEGER(N)[0], INTEGER(P)[0], INTEGER(K)[0], REAL(y),
                        0, REAL(VECTOR_ELT(ret,2)), REAL(VECTOR_ELT(ret,3)), REAL(VECTOR_ELT(ret,4)),
-                       MVT_Nu, (isReal(t) && Rf_length(t) > 0) ? REAL(t) : 0);
+                       MVT_Nu, (Rf_isReal(t) && Rf_length(t) > 0) ? REAL(t) : 0);
         
         status = em.build(INTEGER(label), REAL(VECTOR_ELT(ret,6)),INTEGER(VECTOR_ELT(ret,7)));
         INTEGER(VECTOR_ELT(ret,0))[0] =  INTEGER(K)[0];
@@ -469,7 +471,7 @@ extern "C" {
 		
 		em_mvt em(INTEGER(N)[0], INTEGER(P)[0], INTEGER(K)[0], REAL(y), 
 				  REAL(VECTOR_ELT(ret,1)), REAL(VECTOR_ELT(ret,2)), REAL(VECTOR_ELT(ret,3)), REAL(VECTOR_ELT(ret,4)), 
-				  MVT_Nu, (isReal(t) && Rf_length(t) > 0 ) ? REAL(t) : 0);	
+				  MVT_Nu, (Rf_isReal(t) && Rf_length(t) > 0 ) ? REAL(t) : 0);
 		
 		status = em.start(0);
 		if( status == 0 ) {
@@ -505,7 +507,7 @@ extern "C" {
 		
 		em_mvt em(INTEGER(N)[0], INTEGER(P)[0], INTEGER(K)[0], REAL(y), 
 				  REAL(VECTOR_ELT(ret,1)), REAL(VECTOR_ELT(ret,2)), REAL(VECTOR_ELT(ret,3)), REAL(VECTOR_ELT(ret,4)), 
-				  MVT_Nu, (isReal(t) && Rf_length(t) > 0 ) ? REAL(t) : 0, REAL(bias)[0]);
+				  MVT_Nu, (Rf_isReal(t) && Rf_length(t) > 0 ) ? REAL(t) : 0, REAL(bias)[0]);
 		
 		status = em.start(0);
 		if( status == 0 ) {
@@ -535,7 +537,7 @@ extern "C" {
 
 		em_mvt em(INTEGER(N)[0], INTEGER(P)[0], INTEGER(K)[0], REAL(y), 
 				  REAL(VECTOR_ELT(ret,1)), REAL(VECTOR_ELT(ret,2)), REAL(VECTOR_ELT(ret,3)), REAL(VECTOR_ELT(ret,4)), 
-				  MVT_Nu, (isReal(t) && Rf_length(t) > 0 ) ? REAL(t) : 0);	
+				  MVT_Nu, (Rf_isReal(t) && Rf_length(t) > 0 ) ? REAL(t) : 0);
 		
 		status = em.start(0);
 		if( status == 0 ) {
@@ -573,7 +575,7 @@ extern "C" {
         
         em_mvt2 em(INTEGER(N)[0], INTEGER(P)[0], INTEGER(K)[0], REAL(y),
                   REAL(VECTOR_ELT(ret,1)), REAL(VECTOR_ELT(ret,2)), REAL(VECTOR_ELT(ret,3)), REAL(VECTOR_ELT(ret,4)),
-                  MVT_Nu, (isReal(t) && Rf_length(t) > 0) ? REAL(t) : 0, 0.0);
+                  MVT_Nu, (Rf_isReal(t) && Rf_length(t) > 0) ? REAL(t) : 0, 0.0);
         
         status = em.start(INTEGER(label));
         if( status == 0 ) {
@@ -607,7 +609,7 @@ extern "C" {
         
         em_mvt2 em(INTEGER(N)[0], INTEGER(P)[0], INTEGER(K)[0], REAL(y),
                   REAL(VECTOR_ELT(ret,1)), REAL(VECTOR_ELT(ret,2)), REAL(VECTOR_ELT(ret,3)), REAL(VECTOR_ELT(ret,4)),
-                  MVT_Nu, (isReal(t) && Rf_length(t) > 0 ) ? REAL(t) : 0, REAL(bias)[0]);
+                  MVT_Nu, (Rf_isReal(t) && Rf_length(t) > 0 ) ? REAL(t) : 0, REAL(bias)[0]);
         
         status = em.start(INTEGER(label));
         if( status == 0 ) {
@@ -637,7 +639,7 @@ extern "C" {
         SEXP ret = _M_ret(INTEGER(N)[0], INTEGER(P)[0], INTEGER(K)[0], label);
         em_mvt2 em(INTEGER(N)[0], INTEGER(P)[0], INTEGER(K)[0], REAL(y),
                   0, REAL(VECTOR_ELT(ret,2)), REAL(VECTOR_ELT(ret,3)), REAL(VECTOR_ELT(ret,4)),
-                  MVT_Nu, (isReal(t) && Rf_length(t) > 0) ? REAL(t) : 0);
+                  MVT_Nu, (Rf_isReal(t) && Rf_length(t) > 0) ? REAL(t) : 0);
         
         status = em.build(INTEGER(label), REAL(VECTOR_ELT(ret,6)),INTEGER(VECTOR_ELT(ret,7)));
         INTEGER(VECTOR_ELT(ret,0))[0] =  INTEGER(K)[0];
@@ -663,7 +665,7 @@ extern "C" {
         
         em_mvt2 em(INTEGER(N)[0], INTEGER(P)[0], INTEGER(K)[0], REAL(y),
                   REAL(VECTOR_ELT(ret,1)), REAL(VECTOR_ELT(ret,2)), REAL(VECTOR_ELT(ret,3)), REAL(VECTOR_ELT(ret,4)),
-                  MVT_Nu, (isReal(t) && Rf_length(t) > 0 ) ? REAL(t) : 0);
+                  MVT_Nu, (Rf_isReal(t) && Rf_length(t) > 0 ) ? REAL(t) : 0);
         
         status = em.start(0);
         if( status == 0 ) {
@@ -699,7 +701,7 @@ extern "C" {
         
         em_mvt2 em(INTEGER(N)[0], INTEGER(P)[0], INTEGER(K)[0], REAL(y),
                   REAL(VECTOR_ELT(ret,1)), REAL(VECTOR_ELT(ret,2)), REAL(VECTOR_ELT(ret,3)), REAL(VECTOR_ELT(ret,4)),
-                  MVT_Nu, (isReal(t) && Rf_length(t) > 0 ) ? REAL(t) : 0, REAL(bias)[0]);
+                  MVT_Nu, (Rf_isReal(t) && Rf_length(t) > 0 ) ? REAL(t) : 0, REAL(bias)[0]);
         
         status = em.start(0);
         if( status == 0 ) {
@@ -729,7 +731,7 @@ extern "C" {
         
         em_mvt2 em(INTEGER(N)[0], INTEGER(P)[0], INTEGER(K)[0], REAL(y),
                   REAL(VECTOR_ELT(ret,1)), REAL(VECTOR_ELT(ret,2)), REAL(VECTOR_ELT(ret,3)), REAL(VECTOR_ELT(ret,4)),
-                  MVT_Nu, (isReal(t) && Rf_length(t) > 0 ) ? REAL(t) : 0);
+                  MVT_Nu, (Rf_isReal(t) && Rf_length(t) > 0 ) ? REAL(t) : 0);
         
         status = em.start(0);
         if( status == 0 ) {
@@ -755,19 +757,19 @@ extern "C" {
 	{
 		const int n = INTEGER(N)[0];
 		const int p = INTEGER(P)[0];
-		SEXP ret = Rf_protect(allocVector(VECSXP,3));
-		SEXP names = Rf_protect(allocVector(STRSXP,3));
-		SET_STRING_ELT(names, 0, mkChar("li"));
-		SET_STRING_ELT(names, 1, mkChar("lj"));
-		SET_STRING_ELT(names, 2, mkChar("crit"));
-		SET_VECTOR_ELT(ret, 0, allocVector(INTSXP, n-1));
-		SET_VECTOR_ELT(ret, 1, allocVector(INTSXP, n-1));
-		SET_VECTOR_ELT(ret, 2, allocVector(REALSXP, n-1));
-		setAttrib(ret, R_NamesSymbol, names);
+		SEXP ret = Rf_protect(Rf_allocVector(VECSXP,3));
+		SEXP names = Rf_protect(Rf_allocVector(STRSXP,3));
+		SET_STRING_ELT(names, 0, Rf_mkChar("li"));
+		SET_STRING_ELT(names, 1, Rf_mkChar("lj"));
+		SET_STRING_ELT(names, 2, Rf_mkChar("crit"));
+		SET_VECTOR_ELT(ret, 0, Rf_allocVector(INTSXP, n-1));
+		SET_VECTOR_ELT(ret, 1, Rf_allocVector(INTSXP, n-1));
+		SET_VECTOR_ELT(ret, 2, Rf_allocVector(REALSXP, n-1));
+        Rf_setAttrib(ret, R_NamesSymbol, names);
 		
 		// observation matrix is changed by hc_mvn, so make a copy
 		SEXP x = Rf_protect(Rf_duplicate(y));
-		hc_mvn hc(n, p, REAL(x), (isReal(t) && Rf_length(t) > 0 ) ? REAL(t) : 0);
+		hc_mvn hc(n, p, REAL(x), (Rf_isReal(t) && Rf_length(t) > 0 ) ? REAL(t) : 0);
 		hc.process(INTEGER(VECTOR_ELT(ret,0)), INTEGER(VECTOR_ELT(ret,1)), REAL(VECTOR_ELT(ret,2)));
 		
 		Rf_unprotect(3);
@@ -785,7 +787,7 @@ extern "C" {
 	{
 		vs_htrans ml(INTEGER(N)[0], INTEGER(P)[0], INTEGER(K)[0], REAL(y), REAL(z));
 		
-		SEXP ret = Rf_protect(allocVector(VECSXP,4));
+		SEXP ret = Rf_protect(Rf_allocVector(VECSXP,4));
 		
 		SET_VECTOR_ELT(ret,0, Rf_duplicate(a));
 		SET_VECTOR_ELT(ret,1, Rf_duplicate(b));
@@ -794,12 +796,12 @@ extern "C" {
 		
 		ml.estimate(REAL(VECTOR_ELT(ret,0)), REAL(VECTOR_ELT(ret,1)), INTEGER(VECTOR_ELT(ret,2))[0], REAL(VECTOR_ELT(ret,3))[0], REAL(certainty)[0], method);
 
-		SEXP names = Rf_protect(allocVector(STRSXP,4));
-		SET_STRING_ELT(names, 0, mkChar("a"));
-		SET_STRING_ELT(names, 1, mkChar("b"));
-		SET_STRING_ELT(names, 2, mkChar("iter"));
-		SET_STRING_ELT(names, 3, mkChar("tol"));
-		setAttrib(ret, R_NamesSymbol, names);
+		SEXP names = Rf_protect(Rf_allocVector(STRSXP,4));
+		SET_STRING_ELT(names, 0, Rf_mkChar("a"));
+		SET_STRING_ELT(names, 1, Rf_mkChar("b"));
+		SET_STRING_ELT(names, 2, Rf_mkChar("iter"));
+		SET_STRING_ELT(names, 3, Rf_mkChar("tol"));
+        Rf_setAttrib(ret, R_NamesSymbol, names);
 		
 		Rf_unprotect(2);
 		
@@ -834,7 +836,7 @@ extern "C" {
 	{
 		vs_htrans ml(INTEGER(N)[0], INTEGER(P)[0], INTEGER(K)[0], REAL(y), REAL(z));
 		
-		SEXP ret = Rf_protect(allocVector(VECSXP,4));
+		SEXP ret = Rf_protect(Rf_allocVector(VECSXP,4));
 		
 		SET_VECTOR_ELT(ret,0, Rf_duplicate(a));
 		SET_VECTOR_ELT(ret,1, Rf_duplicate(b));
@@ -843,12 +845,12 @@ extern "C" {
 		
 		ml.estimateA(REAL(VECTOR_ELT(ret,0)), REAL(VECTOR_ELT(ret,1)), INTEGER(VECTOR_ELT(ret,2))[0], REAL(VECTOR_ELT(ret,3))[0], REAL(certainty)[0], method);
 		
-		SEXP names = Rf_protect(allocVector(STRSXP,4));
-		SET_STRING_ELT(names, 0, mkChar("a"));
-		SET_STRING_ELT(names, 1, mkChar("b"));
-		SET_STRING_ELT(names, 2, mkChar("iter"));
-		SET_STRING_ELT(names, 3, mkChar("tol"));
-		setAttrib(ret, R_NamesSymbol, names);
+		SEXP names = Rf_protect(Rf_allocVector(STRSXP,4));
+		SET_STRING_ELT(names, 0, Rf_mkChar("a"));
+		SET_STRING_ELT(names, 1, Rf_mkChar("b"));
+		SET_STRING_ELT(names, 2, Rf_mkChar("iter"));
+		SET_STRING_ELT(names, 3, Rf_mkChar("tol"));
+        Rf_setAttrib(ret, R_NamesSymbol, names);
 		
 		Rf_unprotect(2);
 		
